'use strict';
angular.module('aom.directives')
  .directive('aomMedia', function ($ionicActionSheet, $sce, $ionicModal, $timeout, aomMediaSrv, aomResourcesSrv, aomComponentsSrv, aomAttachmentsSrv, aomPrintingSrv, aomLogSrv, aomPopupSrv) {
    return {
      scope: {
        attachments: '=',
        path: '='
      },
      link: function ($scope) {

        $scope.data = {
          resources: aomResourcesSrv.data.resources
        };

        $scope.checkMediaType = aomMediaSrv.checkMediaType;

        $scope.showOptions = function (attachment) {

          var options = [];
          if (attachment.Source === 'From Device') {
            options.push({ text: aomResourcesSrv.data.resources.AomActionSheetOptions.EditComments, key: 'Edit' });
            options.push({ text: aomResourcesSrv.data.resources.AomActionSheetOptions.DeleteAttachment, key: 'Delete' });
          }
          if ($scope.checkMediaType('image', attachment.MimeType) && aomComponentsSrv.canPrintImages()) {
            options.push({ text: aomResourcesSrv.data.resources.AomActionSheetOptions.Print, key: 'Print' });
          }
          if (options.length > 0) {
            $ionicActionSheet.show({
              buttons: options,
              titleText: aomResourcesSrv.data.resources.AomActionSheetOptions.Title,
              cancelText: aomResourcesSrv.data.resources.Common.Cancel,
              cancel: function () {

              },
              buttonClicked: function (index, object) {
                if (object.key === 'Edit') {
                  aomAttachmentsSrv.editAttachment(attachment.filename, $scope);
                } else if (object.key === 'Delete') {
                  aomAttachmentsSrv.deleteAttachment(attachment.filename)
                    .catch(function (err) {
                      aomPopupSrv.alert('DeleteAttachmentFailed', {
                        title: aomResourcesSrv.data.resources.Common.Error,
                        template: aomResourcesSrv.data.resources.Errors.DeleteAttachmentFailed,
                        onDismiss: function () { }
                      });
                      aomLogSrv.log({ message: 'Delete media attachment.', error: err }, aomLogSrv.levels.user, aomLogSrv.severities.warn);
                    });
                } else if (object.key === 'Print') {
                  aomPrintingSrv.printFile(attachment.filename, attachment.MimeType)
                    .catch(function (err) {
                      aomPopupSrv.alert('PrintingFailed', {
                        title: aomResourcesSrv.data.resources.Common.Error,
                        template: aomResourcesSrv.data.resources.Errors.PrintAttachmentFailed,
                        onDismiss: function () { }
                      });
                      aomLogSrv.log({ message: 'Print attachment failed.', error: err }, aomLogSrv.levels.user, aomLogSrv.severities.err);
                    });
                }
                return true;
              }
            });
          }
        };

      },
      templateUrl: 'App/Common/Directives/Media/media.html'
    };
  });