'use strict';
angular.module('aom.directives')
  .directive('aomSelect', function ($ionicModal, $q, $timeout, $ionicNavBarDelegate, aomResourcesSrv, aomBackSrv, aomStyleSrv, aomPopupSrv, aomSelectModalSrv) {
    return {
      scope: {
        selectObject: '=',
        changedMethod: '=',
        multiSelect: '=',
        selectAllEnabled: '=',
        noClear: '=',
        minimum: '@',
        settings: '@',
        readonly: '='
      },
      link: function ($scope) {

        $scope.isShowingModal = false;

        $scope.data = {
          resources: aomResourcesSrv.data.resources,
          toggleAllDescriptor: aomResourcesSrv.data.resources.Common.SelectAll,
          rowClass: ''
        };

        if (typeof $scope.selectObject.Data === 'object') {
          $scope.selectObject.Data = '';
        }

        if ($scope.selectObject.valid === undefined) {
          $scope.selectObject.valid = true;
        }

        function revertOriginalValues() {
          angular.copy($scope.orignalSelectedParameterValue, $scope.selectObject.SelectedParameterValue);
          angular.copy($scope.originalValues, $scope.selectObject.Parameters.Values);
          angular.copy($scope.originalValue, $scope.selectObject.Data);
        }

        function setData() {
          var newData = '';
          for (var i = 0; i < $scope.selectObject.Parameters.Values.length; i++) {
            if ($scope.selectObject.Parameters.Values[i].Checked) {
              newData += $scope.selectObject.Parameters.Values[i].Text + ', ';
            }
          }
          $scope.selectObject.Data = newData.slice(0, -2);
          if ($scope.selectObject.SelectedParameterValue !== undefined && $scope.multiSelect) {
            angular.copy(Enumerable.from($scope.selectObject.Parameters.Values).where('$.Checked === true').toArray(), $scope.selectObject.SelectedParameterValue);
          }
        }

        function fireChanged() {
          if ($scope.originalValue !== $scope.selectObject.Data) {
            if ($scope.selectObject.changedMethod !== undefined) {
              $scope.selectObject.changedMethod();
            }
            if ($scope.changedMethod !== undefined) {
              $scope.changedMethod($scope.selectObject.Name);
            }
          }
        }

        function backClicked() {
          $scope.isShowingModal = false;
          setData();
          fireChanged();
          aomBackSrv.exit();
          aomSelectModalSrv.hideModal();
        }

        function setupModal() {
          return aomSelectModalSrv.setupModal($scope);
        }

        function toggleSelectAllDescriptor(condition) {
          $scope.data.toggleAllDescriptor = condition ? aomResourcesSrv.data.resources.Common.DeSelectAll : aomResourcesSrv.data.resources.Common.SelectAll;
        }

        $scope.toggleAll = function () {
          $scope.selectObject.Data = '';
          toggleSelectAllDescriptor($scope.data.toggleAllDescriptor === aomResourcesSrv.data.resources.Common.SelectAll);
          for (var i = 0; i < $scope.selectObject.Parameters.Values.length; i++) {
            $scope.selectObject.Parameters.Values[i].Checked = $scope.data.toggleAllDescriptor === aomResourcesSrv.data.resources.Common.DeSelectAll;
          }
          setData();
        };

        $scope.showModal = function () {
          if (!$scope.readonly) {
            aomBackSrv.init($scope.hideModal, 201);
            $scope.isShowingModal = true;
            $scope.originalValue = angular.copy($scope.selectObject.Data);
            $scope.orignalSelectedParameterValue = $scope.selectObject.SelectedParameterValue ? angular.copy($scope.selectObject.SelectedParameterValue) : null;
            $scope.originalValues = angular.copy($scope.selectObject.Parameters.Values);
            $scope.data.rowClass = aomStyleSrv.getRowClass();
            $scope.minimum = $scope.minimum === undefined ? 0 : $scope.minimum;
            $scope.settings = $scope.settings === undefined ? false : $scope.settings;
            setupModal()
              .then(function () {
                $scope.modal.show();
              })
              .then(function () {
                //This is required for single-selects. Below code is redundant for multi-selects
                $scope.selectObject.Data = Enumerable.from($scope.selectObject.Parameters.Values).where('$.Checked === true').toArray()[0];
              });
          }
        };

        $scope.hideModal = function () {
          if ($scope.minimum === 0) {
            backClicked();
          } else {
            var checkedValues = Enumerable.from($scope.selectObject.Parameters.Values).where('$.Checked === true').count();
            if (checkedValues >= $scope.minimum) {
              backClicked();
            } else {
              aomPopupSrv.wrapper({
                id: 'selectMinimumOption',
                title: aomResourcesSrv.data.resources.Common.MinimumSelect + $scope.minimum,
                template: aomResourcesSrv.data.resources.Common.MinimumSelectMessage,
                buttons: [
                  {
                    text: aomResourcesSrv.data.resources.Common.Cancel,
                    type: 'button-stable button-outline',
                    tapMethod: function () {
                      revertOriginalValues();
                      backClicked();
                    }
                  },
                  {
                    text: aomResourcesSrv.data.resources.Common.Okay,
                    type: 'button-positive'
                  }
                ]
              });
            }
          }
        };

        $scope.clearField = function () {
          $scope.selectObject.Data = '';
          for (var i = 0; i < $scope.selectObject.Parameters.Values.length; i++) {
            $scope.selectObject.Parameters.Values[i].Checked = false;
          }
          fireChanged();
        };

        $scope.callClick = function (index, $event) {
          if ($event !== 'change') {
            $scope.selectObject.Parameters.Values[index].Checked = !$scope.selectObject.Parameters.Values[index].Checked;
          }
          if ($event === 'change' || $event.currentTarget.id === 'parent') {
            if (!$scope.multiSelect) {
              for (var i = 0; i < $scope.selectObject.Parameters.Values.length; i++) {
                $scope.selectObject.Parameters.Values[i].Checked = i === index ? $scope.selectObject.Parameters.Values[i].Checked : false;
              }
              if ($scope.selectObject.SelectedParameterValue !== undefined) {
                angular.copy($scope.selectObject.Parameters.Values[index], $scope.selectObject.SelectedParameterValue);
              }
            }
            if ($scope.selectAllEnabled) {
              toggleSelectAllDescriptor($scope.selectObject.Parameters.Values.length === Enumerable.from($scope.selectObject.Parameters.Values).where('$.Checked === true').count());
            }
          }
        };

        $scope.$on('$destroy', function () {
          if ($scope.isShowingModal) {
            setData();
          }
        });

        $scope.$on('modal.hidden', function () {
          if ($scope.isShowingModal) {
            backClicked();
          }
        });

      },
      templateUrl: 'App/Common/Directives/Select/select.html'
    };
  });