'use strict';
angular.module('aom.services')
  .factory('aomPollSrv', function ($timeout, aomAttachmentsSrv, aomJobsSrv, aomNetworkSrv, aomLocationSrv, aomLogSrv, aomLogHandlerSrv, aomSyncSrv, aomResourcesSrv, aomTrackingDataFetchSrv, aomTrackingSrv, aomDeviceSrv, constants) {
    var poll = {},
      defaultPollInterval = 30000,
      logLevel = aomLogSrv.levels.low;

    poll.data = {
      getJobs: {
        enabled: false,
        method: aomJobsSrv.getJobs,
        timeout: null
      },
      sendAttachments: {
        enabled: false,
        method: aomAttachmentsSrv.sendAttachments,
        timeout: null
      },
      sendJobs: {
        enabled: false,
        method: aomJobsSrv.submitJobs,
        timeout: null
      },
      sendAppSettings: {
        enabled: false,
        pollInterval: 3000,
        method: aomSyncSrv.sendAppSettings,
        timeout: null
      },
      sendTrackingGroups: {
        enabled: false,
        pollInterval: 3000,
        method: aomSyncSrv.sendTrackingGroups,
        timeout: null
      },
      getCompanyUsers: {
        enabled: false,
        pollInterval: 30000,
        method: aomTrackingSrv.updateCompanyUsers,
        timeout: null
      },
      getUsersJobs: {
        enabled: false,
        pollInterval: 30000,
        method: aomTrackingDataFetchSrv.getUsersJobs,
        timeout: null
      },
      sendLocation: {
        enabled: false,
        pollInterval: aomResourcesSrv.data.resources.Tracking.Frequency,
        method: aomLocationSrv.sendToServer,
        timeout: null
      },
      housekeepLogs: {
        enabled: false,
        timeoutBeforeFirstCall: true,
        pollInterval: 300000,
        method: aomLogHandlerSrv.housekeep,
        timeout: null
      }
    };

    function callMethodAfterInterval(type) {
      var waitInterval = poll.data[type].pollInterval ? poll.data[type].pollInterval : defaultPollInterval;
      aomLogSrv.log({ message: 'calling method for ' + type + ' with wait interval of ' + waitInterval }, logLevel, aomLogSrv.severities.info);
      poll.data[type].timeout = $timeout(function () {
        callMethod(type);
      }, waitInterval);
    }

    function callMethod(type) {
      if (poll.data[type].enabled) {
        if (aomNetworkSrv.onlineWithRootLinks()) {
          aomLogSrv.log({ message: 'polling for ' + type }, logLevel, aomLogSrv.severities.info);
          poll.data[type].method()
            .finally(function () {
              aomLogSrv.log({ message: 'polling for ' + type + ' recall from finally' }, logLevel, aomLogSrv.severities.info);
              callMethodAfterInterval(type);
            });
        } else {
          aomLogSrv.log({ message: 'polling for ' + type + ' recall' }, aomLogSrv.levels.medium, aomLogSrv.severities.warn);
          callMethodAfterInterval(type);
        }
      }
    }

    poll.startPolling = function (type) {
      aomLogSrv.log({ message: 'Start Polling ' + type }, logLevel, aomLogSrv.severities.info);
      poll.data[type].enabled = true;
      if (poll.data[type].timeoutBeforeFirstCall) {
        callMethodAfterInterval(type);
      } else {
        callMethod(type);
      }
    };

    poll.stopPolling = function (type) {
      aomLogSrv.log({ message: 'Stop Polling ' + type }, logLevel, aomLogSrv.severities.info);
      if (poll.data[type].timeout !== null) {
        $timeout.cancel(poll.data[type].timeout);
      }
      poll.data[type].enabled = false;
    };

    poll.stop = function () {
      constants.polling.map(function (prop) {
        poll.stopPolling(prop);
      });
    };

    poll.start = function () {
      constants.polling.map(function (prop) {
        poll.startPolling(prop);
      });
    };

    return poll;
  });
