'use strict';
angular.module('aom.services')
  .factory('aomResourcesSrv', function ($http, constants) {
    var resources = {};

    resources.data = {
      initialised: false,
      sqliteVersion: 'Unknown',
      applicationsCopy: {},
      normalizedLogo: constants.common.defaultLogo,
      resources: {
        Tracking: {
          StopTrackingAfterMinutes: 480
        },
        Features: {
          IsEnterprise: false
        },
        appConfig: {
          groups: {}
        },
        derived: {
          SelectedTimeout: {
            minutes: 100
          }
        },
        MapOptions: {
          LocationTimeout: 10000,
          LocationMaximumAge: 60000,
          LocationHighAccuracyEnabled: true,
          GoogleFetchingMapInterval: 5000
        },
        LogLevel: 'high',
        Errors: {
          LoginFail: 'Logon failed.',
          DataReset: 'Your data has been reset.',
          NoInternet: 'Cannot communicate with server.',
          IncompleteData: 'Please ensure all data has been entered.',
          Mandatory: 'This data is mandatory.',
          DefaultValidation: 'This data is invalid.',
          EnterpriseFeatureOnly: 'This is Enterprise only feature. Please speak to your administrator for further details'
        },
        Common: {
          Login: 'Login',
          Username: 'Username',
          Password: 'Password',
          Back: 'Back',
          Icons: {
            ErrorInfoLogon: 'ion-information-circled',
            Back: 'ion-ios-arrow-back'
          }
        },
        Logon: {
          TermsAndConditions: 'Terms and conditions',
          TermsAndConditionsMessage: 'This system is for official use only by authorized personnel only. By using this system, you agree to allow monitoring \
          and recording of your activities on this system which may be provided to law enforcement or other investigative agencies for official reasons including \
          criminal investigations. Unauthorized or illegal activities on this system may be punishable by law. By logging into this system you indicate your \
          acceptance of these terms and agree to abide by the IT Acceptable Use Policy.',
          TermsAndConditionsLogonText: 'This system is for official use only by logging on you agree to the '
        },
        Style: {
          ColourScheme: {
            DefaultTextColour: { Data: '#444', DisplayName: 'Default Text Colour' },
            ButtonTextColour: { Data: '#fff', DisplayName: 'Button Text Colour' },
            AssertiveColour: { Data: '#dc3545', DisplayName: 'Assertive Colour' },
            StandardColour: { Data: '#387ef5', DisplayName: 'Highlight Colour' },
            FieldTextColour: { Data: '#287AB4', DisplayName: 'Field Text Colour' },
            BrandColour: { Data: '#1388E3', DisplayName: 'Brand Colour' }
          }
        }
      },
      lastUser: {
        Username: '',
        Password: '',
        failedLoginCount: 0,
        groups: '',
        downloadedLogoLink: null
      },
      dataTapDisabled: false
    };

    resources.setupApplications = function () {
      if (constants.common.applicationResourceName === 'GetApplicationsByUserCompany') {
        resources.data.applicationsCopy = resources.data.resources.appConfig.Tracking;
      } else if (constants.common.applicationResourceName === 'Applications') {
        resources.data.applicationsCopy = resources.data.resources.appConfig;
      }
    };

    $http.get('./appconfig.json')
      .then(function (response) {
        var config = response.data;
        angular.extend(resources.data, {
          firstState: config.mop ? 'app.Templates' : constants.states.firstState,
          MOP: config.mop,
          OEM: config.oem
        });
      });

    return resources;
  });
