angular.module('aom.services')
  .factory('aomTrackingSrv', function ($rootScope, $state, $q, $timeout, leafletMarkersHelpers, aomResourcesSrv, aomComponentsSrv, aomCustomStylesSrv, aomAPISrv, aomJobSrv, aomCleanSrv, aomPopupSrv, aomTrackingHelperSrv) {

    var trackingSrv = {};

    trackingSrv.data = {
      users: {},
      paths: {},
      groups: {},
      selectedGroup: null,
      availableColours: null,
      layerBounds: {
        userMarkers: []
      },
      userSearchQuery: '',
      modeState: {
        users: {},
        markers: {}
      },
      layerColours: { // TODO move to helper
        jobHistory: {}
      },
      modeStateUsersUpdating: false
    };

    function iconCreate(cluster) {
      var layer = trackingSrv.data.modeState.markers[cluster.getAllChildMarkers()[0].options.id].layer,
        colour = layer;
      if (trackingSrv.data.modeState.groupMode) {
        if (trackingSrv.data.selectedGroup.users[layer] !== undefined) {
          colour = trackingSrv.data.selectedGroup.users[layer].colour;
        }
      } else {
        if (trackingSrv.data.layerColours.jobHistory[layer] !== undefined) {
          colour = trackingSrv.data.layerColours.jobHistory[layer];
        } else if (layer !== 'userMarkers') {
          if (trackingSrv.data.modeState.selectedUser !== null) {
            colour = trackingSrv.data.modeState.selectedUser.colour;
          }
        }
      }
      return new L.DivIcon({
        html: '<div style="top: -14px; left: -14px; position: relative; opacity: 0.7;"><span style="color: white;">' + cluster.getAllChildMarkers().length + '</span></div>',
        className: 'marker-cluster marker-cluster-small colour-' + colour
      });
    }

    function forEachUserInSelectedGroup(method, forDeletedUsers) {
      if (trackingSrv.data.selectedGroup !== null) {
        for (var prop in trackingSrv.data.selectedGroup.users) {
          if (trackingSrv.data.selectedGroup.users.hasOwnProperty(prop) && (forDeletedUsers || !trackingSrv.data.selectedGroup.users[prop].deleted)) {
            method(prop);
          }
        }
      }
    }

    function changeUsersJobsColour(userId, colourName) {
      for (var prop in trackingSrv.data.modeState.markers) {
        if (trackingSrv.data.modeState.markers.hasOwnProperty(prop) && trackingSrv.data.modeState.markers[prop].layer === userId) {
          trackingSrv.data.modeState.markers[prop].icon.className = 'awesome-marker-shadow markercolour-' + colourName;
        }
      }
    }

    function userHasSelectedApplications(user) {
      return user.Applications.filter(function (application) {
        return trackingSrv.data.selectedApplicationReferences[application.Reference];
      }).length !== 0;
    }

    function filterUsersBySelectedApplications() {
      var users = angular.copy(trackingSrv.data.users);
      var filteredList = {};

      for (var user in users) {
        if (users.hasOwnProperty(user) && userHasSelectedApplications(users[user])) {
          filteredList[user] = users[user];
        }
      }
      return filteredList;
    }

    function updateUserMarker(markerId, user) {
      trackingSrv.data.modeState.markers[markerId].lat = user.LastKnownLocation.Latitude;
      trackingSrv.data.modeState.markers[markerId].lng = user.LastKnownLocation.Longitude;
      trackingSrv.data.modeState.markers[markerId].message = user.message;
    }

    function updateUser(modifyUser, user) {
      var markerId = trackingSrv.data.modeState.selectedUser && trackingSrv.data.modeState.selectedUser.Id === user.Id ? 'selectedUser' : user.Id;
      modifyUser.LastKnownLocation = user.LastKnownLocation;
      modifyUser.message = user.Username;
      updateUserMarker(markerId, user);
      return modifyUser;
    }

    function createUser(user) {
      user.inGroup = trackingSrv.data.selectedGroup.users[user.Id] !== undefined;
      user.jobLayer = trackingSrv.createLayer(user.Id, false);
      trackingSrv.generateMarker(user.Id, 'userMarkers', user.LastKnownLocation.Latitude, user.LastKnownLocation.Longitude, user.Username, 1, 'userMarkers', '<i class="ion-ios-person user-icon"></i>');
      return user;
    }

    function updateModeStateUserFlags(id) {
      trackingSrv.data.modeState.users[id].inGroup = false;
      trackingSrv.data.modeState.users[id].showJobs = false;
    }

    function removeSelectedUserFromMap(overlays, id) {
      updateModeStateUserFlags(id);
      if (overlays) { overlays[id].visible = false; }
      if (trackingSrv.data.modeState.markers[id]) {
        trackingSrv.data.modeState.markers[id].layer = 'userMarkers';
        trackingSrv.data.modeState.markers[id].icon.className = 'awesome-marker-shadow  markercolour-userMarkers';
      }
    }

    function makeUserMarkerGroupMarker(prop) {
      changeUsersJobsColour(prop, trackingSrv.data.selectedGroup.users[prop].colour);
      if (trackingSrv.data.modeState.markers[prop]) {
        trackingSrv.data.modeState.markers[prop].icon.className = 'awesome-marker-shadow markercolour-' + trackingSrv.data.selectedGroup.users[prop].colour;
        trackingSrv.data.modeState.markers[prop].layer = 'groupUserMarkers';
      }
    }

    trackingSrv.generateMarker = function (key, layer, lat, lng, message, opacity, colour, icontemplate) {
      if ((lat !== 0 && lng !== 0) || (key === 'selectedUser')) {
        trackingSrv.data.layerBounds[layer] === undefined ? trackingSrv.data.layerBounds[layer] = [[lat, lng]] : trackingSrv.data.layerBounds[layer].push([lat, lng]);
        trackingSrv.data.modeState.markers[key] = aomTrackingHelperSrv.generateMarkerObject(key, layer, lat, lng, message, opacity, colour, icontemplate);
      }
    };

    trackingSrv.resetAvailableColours = function () {
      trackingSrv.data.availableColours = angular.copy(aomResourcesSrv.data.resources.Tracking.UserColours);
      //TODO: Remove after the resources on the server are changed
      if (trackingSrv.data.availableColours.dev) {
        delete trackingSrv.data.availableColours.dev;
      }
      if (trackingSrv.data.availableColours.dev2) {
        delete trackingSrv.data.availableColours.dev2;
      }
      if (trackingSrv.data.availableColours.userMarkers) {
        delete trackingSrv.data.availableColours.userMarkers;
      }
      forEachUserInSelectedGroup(function (prop) {
        delete trackingSrv.data.availableColours[trackingSrv.data.selectedGroup.users[prop].colour];
      });
    };

    trackingSrv.updateModeStateUsers = function () {
      var users = filterUsersBySelectedApplications();
      var oldUsers = angular.copy(trackingSrv.data.modeState.users);
      for (var prop in users) {
        if (users.hasOwnProperty(prop)) {
          var user = users[prop];
          trackingSrv.data.modeState.users[user.Id] = oldUsers[user.Id] !== undefined ? updateUser(oldUsers[user.Id], user) : createUser(user);
        }
      }
      trackingSrv.updateSelectedGroupUsers();
    };

    // We are not removing users that are deleted or changed to not tracked until we return to the mode screen
    trackingSrv.resetCompanyUsers = function () {
      if (trackingSrv.data.masterUsers) {
        angular.copy(trackingSrv.data.masterUsers, trackingSrv.data.users);
        trackingSrv.updateModeStateUsers();
      }
    };

    function handleUsersResponse(response) {
      var users = {};
      for (var i = 0; i < response.Users.length; i++) {
        response.Users[i].Id = 'user' + response.Users[i].Id;
        if (response.Users[i].LastKnownLocation !== null && response.Users[i].Applications.length !== 0 && response.Users[i].Claims.IsTrackedUser) {
          users[response.Users[i].Id] = response.Users[i];
          trackingSrv.data.users[response.Users[i].Id] = response.Users[i];
        }
      }
      trackingSrv.data.masterUsers = users;
      trackingSrv.updateModeStateUsers();
    }

    trackingSrv.updateCompanyUsers = function () {
      return aomAPISrv.getData(aomAPISrv.data.rootLinks.GetMyCompaniesUsers.href)
        .then(function (response) {
          return trackingSrv.processIfModeStateUsersStatic(handleUsersResponse, response);
        })
        .then(function () {
          trackingSrv.data.modeStateUsersUpdating = false;
          return $q.when();
        });
    };

    trackingSrv.updateSelectedGroupUsers = function () {
      var allUsers = Object.keys(trackingSrv.data.users);
      Object.keys(trackingSrv.data.selectedGroup.users).map(function (groupUser) {
        trackingSrv.data.selectedGroup.users[groupUser].deleted = allUsers.indexOf(groupUser) === -1;
        if (trackingSrv.data.modeState.groupMode) {
          makeUserMarkerGroupMarker(groupUser);
        }
      });
    };

    trackingSrv.generateMarkerScope = function (key, job) {
      trackingSrv.data.modeState.markers[key].getMessageScope = function () {
        var scope = $rootScope.$new();
        scope.job = job;
        scope.openJob = function (job) {
          aomJobSrv.data.selectedJob = job;
          aomJobSrv.data.initComplete = false;
          aomJobSrv.data.features = {
            readonly: true,
            linkToBridge: true,
            runSql: false,
            canSubmit: false
          };
          if (aomJobSrv.data.selectedJob) {
            aomCleanSrv.cleanBeforeJobFetch(aomJobSrv.data.selectedJob);
            $state.go('Job');
          } else {
            aomPopupSrv.alert('NoJobData', {
              title: aomResourcesSrv.data.resources.Common.Error,
              template: 'No job data',
              onDismiss: function () { }
            });
          }
        };
        return scope;
      };
    };

    trackingSrv.createLayer = function (name, visible, type) {
      return {
        name: '' + name,
        type: type ? type : 'markercluster',
        visible: visible,
        layerOptions: {
          disableClusteringAtZoom: 18,
          singleMarkerMode: false,
          showCoverageOnHover: false,
          iconCreateFunction: iconCreate
        },
        groupOption: {
          spiderfyDistanceMultiplier: 1.5
        }
      };
    };

    trackingSrv.addGroup = function (overlays) {
      var groupId = aomComponentsSrv.generateUniqueId();
      trackingSrv.data.groups[groupId] = {
        id: groupId,
        isEmpty: true,
        name: 'Group ' + Object.keys(trackingSrv.data.groups).length,
        users: {}
      };
      trackingSrv.changeGroup(trackingSrv.data.groups[groupId], overlays);
    };

    trackingSrv.removeSelectedGroupFromMap = function (overlays) {
      forEachUserInSelectedGroup(function (prop) {
        removeSelectedUserFromMap(overlays, prop);
      });
    };

    trackingSrv.addSelectedGroupToMap = function (overlays) {
      forEachUserInSelectedGroup(function (prop) {
        if (trackingSrv.data.modeState.users[prop]) { trackingSrv.data.modeState.users[prop].inGroup = true; }
        trackingSrv.data.selectedGroup.users[prop].deleted = !trackingSrv.data.modeState.users[prop];
        if (overlays && !trackingSrv.data.selectedGroup.users[prop].deleted) {
          makeUserMarkerGroupMarker(prop);
        }
      });
      if (overlays) {
        forEachUserInSelectedGroup(function (prop) {
          var groupUser = trackingSrv.data.selectedGroup.users[prop];
          if (!overlays[prop]) { overlays[prop] = trackingSrv.data.selectedGroup.users[prop].jobLayer; }
          overlays[prop].visible = !!groupUser.showJobs && trackingSrv.data.modeState.activeJobs && !groupUser.deleted;
        }, true);
      }
      trackingSrv.resetAvailableColours();
    };

    trackingSrv.changeGroup = function (group, overlays) {
      if (trackingSrv.data.selectedGroup !== null) {
        trackingSrv.removeSelectedGroupFromMap(overlays);
      }
      trackingSrv.data.selectedGroup = group;
      trackingSrv.addSelectedGroupToMap(overlays);
    };

    trackingSrv.addUserToGroup = function (colourName, user) {
      user.showColourPicker = false;
      user.colour = colourName;
      user.inGroup = true;
      trackingSrv.data.selectedGroup.users[user.Id] = angular.copy(user);
      trackingSrv.data.selectedGroup.isEmpty = false;
      delete trackingSrv.data.availableColours[colourName];
    };

    trackingSrv.removeUserFromGroup = function (user) {
      updateModeStateUserFlags(user.Id);
      delete trackingSrv.data.selectedGroup.users[user.Id];
      trackingSrv.data.selectedGroup.isEmpty = Object.keys(trackingSrv.data.selectedGroup.users).length === 0;
      trackingSrv.resetAvailableColours();
    };

    trackingSrv.hideSelectedUserHistory = function (overlays, user) {
      for (var key in user.history) {
        if (user.history.hasOwnProperty(key)) {
          user.history[key].checked = false;
          user.history[key].timePosition = 1;
          if (overlays && overlays[key + user.Id]) {
            overlays[key + user.Id].visible = false;
          }
        }
      }
    };

    trackingSrv.updateSelectedUser = function (user, overlays) {
      if (trackingSrv.data.modeState.selectedUser) {
        overlays[trackingSrv.data.modeState.selectedUser.Id].visible = false;
        trackingSrv.hideSelectedUserHistory(overlays, trackingSrv.data.modeState.selectedUser);
        trackingSrv.data.modeState.markers[trackingSrv.data.modeState.selectedUser.Id] = trackingSrv.data.modeState.selectedUser.marker;
      }
      overlays.selectedUser.visible = trackingSrv.data.modeState.activeJobs;
      updateUserMarker('selectedUser', user);
      if (user.history === undefined) { user.history = {}; }
      trackingSrv.data.modeState.selectedUser = user;
      trackingSrv.data.modeState.selectedUser.marker = trackingSrv.data.modeState.markers[user.Id];
      delete trackingSrv.data.modeState.markers[user.Id];
    };

    trackingSrv.processIfModeStateUsersStatic = function (method, user, data) {
      if (trackingSrv.data.modeStateUsersUpdating) {
        return $timeout(function () {
          return trackingSrv.processIfModeStateUsersStatic(method, user, data);
        }, 100);
      } else {
        trackingSrv.data.modeStateUsersUpdating = true;
        return method(user, data);
      }
    };

    // method set by controller
    trackingSrv.fitBounds = function () { };

    return trackingSrv;
  });