'use strict';
angular.module('aom.controllers')
  .controller('DebugCtrl', function ($scope, $state, $q, aomCustomUriSrv, $cordovaFile, aomFileSystemSrv, aomAPISrv, aomPopupSrv, aomComponentsSrv, constants, aomUserSrv, aomResourcesSrv) {
    var uniqueId = (new Date()).toISOString().replace(/:|-|\./g, ''),
      sentFiles = [];

    aomPopupSrv.closePopups();

    if (aomResourcesSrv.data.lastUser.Username.length > 0 && aomResourcesSrv.data.lastUser.Password.length > 0) {
      aomUserSrv.logonRequest.Password = aomResourcesSrv.data.lastUser.Password;
      aomUserSrv.logonRequest.Username = aomResourcesSrv.data.lastUser.Username;
      aomUserSrv.logon();
    } else {
      alert('Can\'t send logs for unauthenticated user');
    }

    $scope.data = {
      systemUpload: aomCustomUriSrv.data.systemUpload,
      logs: {
        media: {
          path: aomFileSystemSrv.applicationDataPath,
          files: [],
          isPressed: false,
          isSent: false,
          messages: {
            success: 'Media files have been sent to server.',
            failure: 'We are unable to media files at this time.'
          }
        },
        logs: {
          path: `${cordova.file.dataDirectory}${constants.common.logsFolder}`,
          files: [],
          isPressed: false,
          isSent: false,
          messages: {
            success: 'Log files have been sent to server.',
            failure: 'We are unable to send log files at this time.'
          }
        },
        database: {
          path: aomComponentsSrv.databasePath(),
          files: [],
          isPressed: false,
          isSent: false,
          messages: {
            success: 'Database have been sent to server.',
            failure: 'We are unable to database at this time.'
          }
        }
      }
    };

    function getLogsPath() {
      for (const key in $scope.data.logs) {
        if ($scope.data.logs.hasOwnProperty(key)) {
          const element = $scope.data.logs[key];
          getFiles(element.path, element.files);
        }
      }
    }

    function getFiles(path, files) {
      aomFileSystemSrv.listFilenames(path)
        .then(aFiles => {
          aFiles.forEach(file => {
            if (file.isDirectory) {
              getFiles(file.nativeURL, files);
            } else if (file.isFile) {
              files.push({ name: file.name, path: path });
            }
          });
        });
    }

    function sendFileToServer(file) {
      if (sentFiles.indexOf(file.name) === -1) {
        return aomAPISrv.fileUpload(aomFileSystemSrv.normalizeURL(file.path), file.name, 'application/octet-stream', null, null, 'debug', uniqueId)
          .then(function (filename) {
            sentFiles.push(filename);
            return $q.when();
          });
      }
    }

    getLogsPath();

    $scope.back = aomCustomUriSrv.back;

    $scope.send = function send(type) {
      if (!$scope.data.logs[type].isPressed) {
        $scope.data.logs[type].isPressed = true;
        const promises = $scope.data.logs[type].files.map(file => sendFileToServer(file));
        $q.all(promises)
          .then(function () {
            alert($scope.data.logs[type].messages.success);
            $scope.data.logs[type].isSent = true;
          })
          .catch(function () {
            alert($scope.data.logs[type].messages.failure);
          })
          .finally(function () {
            $scope.data.logs[type].isPressed = false;
          });
      }
    };

    $scope.$on('$ionicView.beforeEnter', function (event, viewData) {
      viewData.enableBack = true;
    });

  });