﻿'use strict';
angular.module('aom.directives')
  .directive('aomJob', function ($ionicLoading, $rootScope, $q, $location, $state, $timeout, $ionicActionSheet, $ionicScrollDelegate, aomAttachmentsSrv, constants, aomJobSrv, aomLocationSrv,
    aomComponentsSrv, aomResourcesSrv, aomBackSrv, aomJobsSrv, aomPopupSrv, aomGoogleMapsSrv, aomLogSrv, aomAutoSaveSrv, aomStyleSrv, aomMediaSrv, leafletData, aomCleanSrv, aomAPISrv, aomDeviceSrv, aomJobHelperSrv, aomStateSrv, aomLogoutSrv, aomSelectModalSrv, aomSubmitCheckSrv) {
    return {
      scope: {},
      controller: function ($scope) {
        var removeJobWatch;
        var setFlagsTimeout = null;
        var fieldTypes = {};

        for (var prop in constants.fieldTypes) {
          if (constants.fieldTypes.hasOwnProperty(prop)) {
            fieldTypes[constants.fieldTypes[prop]] = 'field-' + prop;
          }
        }

        $scope.$on(constants.broadcastEvents.selectJob, function () {
          $scope.data.currentState = constants.viewStates.job;
          aomJobsSrv.setupDraftJobs();
          aomBackSrv.init($scope.backClicked);
          setCalcFlags();
        });

        var deregister = null;

        if (aomStateSrv.data.main === 'jobs') {
          deregister = $rootScope.$on('backToJobs', backToJobs);
        }

        this.$onInit = function () {
          aomJobsSrv.setupDraftJobs();
          aomBackSrv.init($scope.backClicked);
        };

        $scope.toggleFullscreen = function toggleFullscreen() {
          $scope.data.viewstate = $scope.data.viewstate === 'expand' ? 'compress' : 'expand';
          aomStateSrv.toggleFullscreen();
          if ($scope.data.currentState === constants.viewStates.map) {
            $scope.$broadcast('ReloadJobMap');
          }
        };

        $scope.getUIClasses = function (field) {
          var arr = [
            $scope.data.rowClass,
            getFieldTypeText(field.Type),
            $scope.isDataEmpty(field.DisplayName) ? 'empty' : '',
            field.Options.IsReadOnly ? 'readonly' : ''
          ];
          return arr;
        };

        function getFieldTypeText(index) {
          return fieldTypes[index];
        }

        $scope.isDataEmpty = function (value) {
          return (!value || value.length === 0);
        };

        $scope.data = {
          aomJobSrvData: aomJobSrv.data,
          resources: aomResourcesSrv.data.resources,
          mapData: aomGoogleMapsSrv.data,
          constants: constants,
          animation: 'next-page',
          rowClass: 'row-spacing-md',
          pageValid: false,
          paddingHeight: ionic.Platform.isIOS() ? (window.screen.height - 200) / 2 + 'px' : null,
          hasPaging: aomJobSrv.data.selectedJob.Paging.Pages.length > 1,
          limit: 25,
          currentState: constants.viewStates.job,
          viewstate: 'expand'
        };

        $scope.shared = {
          keyboardUp: false
        };

        var jobMap = null;
        angular.extend($scope.data, {
          showMap: true,
          bounds: [],
          map: {
            defaults: {
              maxZoom: 16,
              minZoom: 3,
              zoomAnimation: true
            },
            layers: {
              baselayers: angular.copy(aomResourcesSrv.data.resources.Tracking.baselayers),
              overlays: {}
            }
          },
          mapPosition: { 'flex-direction': 'row-reverse' }
        });

        for (var key in $scope.data.map.layers.baselayers) {
          if ($scope.data.map.layers.baselayers.hasOwnProperty(key)) {
            $scope.data.map.layers.baselayers[key].top = aomResourcesSrv.data.resources.derived.MapViewSelected.Code === $scope.data.map.layers.baselayers[key].layerType;
          }
        }

        $scope.data.map.layers.overlays.jobMarkers = aomJobHelperSrv.createLayer('jobMarkers', true);

        $scope.submitUpdate = function () {
          submitAndSetStatus(constants.status.updating);
        };

        function initJob() {
          if (!aomJobSrv.data.initComplete) {
            aomAPISrv.getData(aomJobSrv.data.selectedJob.links.self.href)
              .then(function (job) {
                return aomCleanSrv.cleanJob(job);
              })
              .then(function (job) {
                aomJobSrv.data.changesMade = false;
                removeJobWatch();

                aomJobSrv.data.selectedJob = angular.copy(job);
                if (window.cordova && aomResourcesSrv.data.resources.Tracking.DownloadAttachmentsOnJobOpen) {
                  aomJobHelperSrv.getAttachments();
                }
                aomJobHelperSrv.getLocations();
                jobMap.fitBounds(aomJobSrv.data.selectedJob.bounds, { maxZoom: 16, padding: [50, 50] });
                aomJobSrv.setupJob();
              })
              .catch(function () {
                backToJobs();
                aomPopupSrv.alert('JobLoadFailed', {
                  title: aomResourcesSrv.data.resources.Common.Error,
                  template: 'There was an error loading the job',
                  onDismiss: function () { }
                });
              });
          }
        }

        $scope.init = function () {
          leafletData.getMap('jobmap')
            .then(function (map) {
              jobMap = map;
              map.worldCopyJump = false;
              map.removeControl(map.attributionControl);
              map.setMaxBounds([
                [-90, -180],
                [90, 180]
              ]);
              if (aomJobSrv.data.selectedJob.bounds && aomJobSrv.data.selectedJob.bounds.length > 0) {
                jobMap.fitBounds(aomJobSrv.data.selectedJob.bounds, { maxZoom: 16, padding: [50, 50] });
              }
            });
        };

        $scope.data.readonly = aomJobSrv.data.features.readonly;

        $scope.data.rowClass = aomStyleSrv.getRowClass();

        function isScrollingRequired() {
          return $ionicScrollDelegate.$getByHandle('job-table').getScrollPosition().top !== 0;
        }

        function scrollToSelectedField() {
          var scrollPosition = aomJobSrv.data.selectedFieldPosition;
          if (scrollPosition) {
            $timeout(function () {
              $ionicScrollDelegate.$getByHandle('job-table').scrollTo(scrollPosition.left, scrollPosition.top, true);
            });
          }
        }

        function scrollToTop() {
          if (isScrollingRequired()) {
            $timeout(function () {
              $ionicScrollDelegate.$getByHandle('job-table').scrollTop(true);
            }, 100);
          }
        }

        function setAttachmentSrvData(attachments, options) {
          aomAttachmentsSrv.data.attachments = attachments;
          aomAttachmentsSrv.data.options = options;
        }

        function checkMandatoryAndValid() {
          var checkPassed = true;
          var pageFields = Enumerable
            .from(aomJobSrv.data.selectedJob.Fields)
            .where('$.PageName === "' + aomJobSrv.data.selectedJob.Paging.selectedPage + '" && $.visible')
            .toArray();
          for (var i = 0; i < pageFields.length; i++) {
            if (!pageFields[i].valid || (pageFields[i].Options.IsRequired && (pageFields[i].Data === null || pageFields[i].Data.length === 0))) {
              checkPassed = false;
            }
          }
          aomJobSrv.data.selectedJob.valid = checkPassed;
          return checkPassed;
        }

        function saveAndExit() {
          removeJobWatch();
          aomAutoSaveSrv.cancelSaveTempJob();
          saveJob()
            .then(function () {
              aomJobsSrv.submitJobs();
              exitJobScreen();
            })
            .catch(function (err) {
              aomPopupSrv.alert('jobSaveFailed', {
                title: aomResourcesSrv.data.resources.Common.Error,
                template: aomResourcesSrv.data.resources.Job.FailedToSave,
                onDismiss: function () { }
              });
              aomLogSrv.log({ message: 'failed in job.ctrl.saveAndExit', error: err }, aomLogSrv.levels.user, aomLogSrv.severities.err);
            });
        }

        function setLocationWhilstSubmitting(status) {
          if (status === constants.status.pending) {
            return aomLocationSrv.getCurrentLocation()
              .then(function () {
                aomJobSrv.data.selectedJob.CustomLocations.push(
                  {
                    Latitude: aomLocationSrv.data.lastLocationForServer.latitude,
                    Longitude: aomLocationSrv.data.lastLocationForServer.longitude,
                    Comments: aomResourcesSrv.data.resources.Job.LocationWhilstSubmitting,
                    DateTime: moment().format()
                  }
                );
                aomJobSrv.data.selectedJob.Submitted = {
                  DateTime: moment().format(),
                  Location:
                  {
                    Longitude: aomLocationSrv.data.lastLocationForServer.longitude,
                    Latitude: aomLocationSrv.data.lastLocationForServer.latitude,
                    Altitude: aomLocationSrv.data.lastLocationForServer.altitude,
                    Accuracy: aomLocationSrv.data.lastLocationForServer.accuracy,
                    AltitudeAccuracy: aomLocationSrv.data.lastLocationForServer.altitudeAccuracy,
                    Heading: aomLocationSrv.data.lastLocationForServer.heading,
                    Speed: aomLocationSrv.data.lastLocationForServer.speed
                  }
                };
                return $q.when();
              });
          } else {
            return $q.when();
          }
        }

        function submitAndSetStatus(status) {
          $ionicLoading.show().then(function () {
            if (status === constants.status.pending) {
              aomLogSrv.log({ message: 'Submit Selected' }, aomLogSrv.levels.medium, aomLogSrv.severities.info);
            }
            aomJobSrv.data.selectedJob.readyToSubmit = true;
            aomJobSrv.data.selectedJob.jsProperties.status = status;
            setLocationWhilstSubmitting(status)
              .then(function () {
                return aomJobsSrv.removeFromDrafts(aomJobSrv.data.selectedJob.Id);
              })
              .then(function () {
                saveAndExit();
              });
          });
        }

        function saveJob() {
          var originalId = angular.copy(aomJobSrv.data.selectedJob.Id);
          var wasTemplate = aomJobSrv.data.selectedJob.isTemplate;
          if (aomJobSrv.data.selectedJob.isTemplate) {
            aomJobSrv.data.selectedJob.Id = aomJobSrv.data.selectedJob.newId;
            aomJobSrv.data.selectedJob.isTemplate = false;
            aomJobSrv.data.selectedJob.fromTemplate = true;
          }
          aomJobSrv.data.selectedJob.DateModified = moment().format();
          aomLogSrv.log({ message: 'Attempting to save job with Job Id ' + aomJobSrv.data.selectedJob.Id + ' and title ' + aomJobSrv.data.selectedJob.Title }, aomLogSrv.levels.high, aomLogSrv.severities.info);
          return aomJobsSrv.insertJob(aomJobSrv.data.selectedJob)
            .then(function () {
              return aomAttachmentsSrv.commitAttachmentUpdates(originalId);
            })
            .then(function () {
              return aomAttachmentsSrv.updateJobId(originalId, aomJobSrv.data.selectedJob.newId, wasTemplate);
            })
            .then(function () {
              return aomAutoSaveSrv.completeSaveJob(originalId, wasTemplate);
            });
        }

        function removeAdhocAttachments() {
          if (aomJobSrv.data.downloadedAdhocAttachment) {
            aomDeviceSrv.removeAdhocDocumentsAndRecreateDirectory();
          }
        }

        function exitJobScreen() {
          $ionicLoading.hide()
            .then(function () {
              removeAdhocAttachments();
              aomBackSrv.exit(true);
              aomJobsSrv.data.selectedJobId = null;
              aomJobsSrv.data.selectedMapJobId = null;

              if ($scope.data.viewstate === 'compress') {
                $scope.toggleFullscreen();
              }

              aomStateSrv.setRight(constants.viewStates.empty);
            });
        }

        function backToJobs() {
          if (aomJobSrv.data.changesMade) {
            showIonicPopUp(aomResourcesSrv.data.resources.Common.Save, aomResourcesSrv.data.resources.Job.SaveMessage, false);
          } else {
            exitJobScreen();
          }
        }

        $scope.backClicked = function backClicked() {
          if (aomJobSrv.data.selectedJob.Paging.pageHistory.length !== 0) {
            $scope.data.animation = 'previous-page';
            aomJobSrv.data.selectedJob.Paging.selectedPage = aomJobSrv.data.selectedJob.Paging.pageHistory.pop();
            setCalcFlags();
            aomJobSrv.setShowSubmitFlags();
            scrollToTop();
          } else {
            backToJobs();
          }
        };

        function showIonicPopUp(title, template, isSubmit) {
          var buttonsOnPopUp = [
            {
              text: aomResourcesSrv.data.resources.Common.Cancel,
              type: 'button-stable button-outline',
              tapMethod: () => {
                aomSubmitCheckSrv.data.submitting = false;
              }
            },
            {
              text: aomResourcesSrv.data.resources.Common.No,
              type: 'button-stable button-outline',
              tapMethod: function () {
                aomSubmitCheckSrv.data.submitting = false;
                if (!isSubmit) {
                  removeJobWatch();
                  aomAutoSaveSrv.cancelSaveTempJob();
                  aomAutoSaveSrv
                    .discardJob()
                    .then(function () {
                      if (aomJobSrv.data.selectedJob.isTemplate) {
                        aomAttachmentsSrv.deleteAttachmentFromDatabaseByJobId(aomAutoSaveSrv.data.selectedJob.Id);
                      }
                      exitJobScreen();
                    });
                }
              }
            },
            {
              text: aomResourcesSrv.data.resources.Common.Yes,
              type: 'button-positive',
              tapMethod: function () {
                aomSubmitCheckSrv.data.submitting = false;
                aomSelectModalSrv.hideModal();
                isSubmit ? submitAndSetStatus(constants.status.pending) : saveAndExit();
              }
            },
          ];
          if (isSubmit) {
            buttonsOnPopUp.shift();
          }
          aomPopupSrv.wrapper({
            id: 'jobGeneric',
            title: title,
            template: template,
            cssClass: 'large-popup',
            buttons: buttonsOnPopUp
          });
        }

        function setCalcFlags() {
          if (!$scope.data.readonly) {
            if (setFlagsTimeout !== null) {
              $timeout.cancel(setFlagsTimeout);
            }
            setFlagsTimeout = $timeout(function () {
              $scope.data.pageValid = !aomJobSrv.data.aomSqlMapSrvData.isSQLRunning && checkMandatoryAndValid();
            }, 40);
          }
        }

        aomJobSrv.data.activeJobId = aomJobSrv.data.selectedJob.Id;

        $scope.calc = aomJobSrv.calc;
        $scope.setCalcFlags = setCalcFlags;

        $scope.showMap = aomJobSrv.showMap;
        $scope.checkMediaType = aomMediaSrv.checkMediaType;
        $scope.makeTrustedURL = aomComponentsSrv.makeTrustedURL;

        $scope.changeToFieldController = function (field, stateName) {
          var options = { Title: field.DisplayName, isField: true };
          if (field.Type === aomJobSrv.fieldTypes.photoVideo) {
            options.Type = field.Options.CameraType;
          } //We do this because we have an attachment field as well which seems to be a read only field (don't ask why)

          setAttachmentSrvData(field.Attachments, options);
          aomJobSrv.setField(field);

          $state.go(stateName);
        };

        $scope.scrollTo = function (target) {
          if (ionic.Platform.isIOS()) {
            $location.hash(target);
            $ionicScrollDelegate.$getByHandle('job-table').anchorScroll(false);
          }
        };

        $scope.showCurrentLatAndLong = function (field) {
          aomLocationSrv.getCurrentLocation()
            .then(function () {
              field.Data = aomLocationSrv.data.lastLocationForServer.latitude + ',' + aomLocationSrv.data.lastLocationForServer.longitude;
              $scope.calc(field.Name);
            });
        };

        $scope.selectField = function () {
          aomLogoutSrv.updateLastActivity();
          aomJobSrv.data.selectedFieldPosition = $ionicScrollDelegate.$getByHandle('job-table').getScrollPosition();
        };

        $scope.invalidOrMandatoryClick = function ($event) {
          $timeout(function (element) {
            angular.element(element.previousElementSibling.children[0]).triggerHandler('click');
          }, 500, true, $event.currentTarget);
        };

        $scope.recordGeoCoordAndDateTime = function (field) {
          if (field.Options.RecordGeoCoordinates) {
            // TODO: Have a look at this later
            if (field.Type === aomJobSrv.fieldTypes.counter) {
              field.CustomLocations.length = 0;
            }
            if (aomResourcesSrv.data.resources.appConfig.RecordGeoLocations) {
              aomLocationSrv.getAccurateCurrentLocation()
                .then(function () {
                  field.CustomLocations.push({
                    Latitude: aomLocationSrv.data.lastLocationForServer.latitude,
                    Longitude: aomLocationSrv.data.lastLocationForServer.longitude,
                    DateTime: moment().format()
                  });
                });
            } else {
              field.CustomLocations.push({
                Latitude: null,
                Longitude: null,
                DateTime: moment().format()
              });
            }
          }
        };

        function revertJob() {
          return aomJobsSrv.removeFromDrafts(aomJobSrv.data.selectedJob.Id)
            .then(function () {
              return aomJobSrv.revert();
            })
            .then(function () {
              setCalcFlags();
            });
        }

        function showJobOptions() {

          var options = [
            { text: aomResourcesSrv.data.resources.Job.LinkToBridge, key: 'BridgeLink', show: aomJobSrv.data.features.linkToBridge },
            { text: '<div class="' + ($scope.data.pageValid ? 'submit' : 'nosubmit') + '">' + aomResourcesSrv.data.resources.Common.Submit + '</div>', key: 'Submit', show: aomJobSrv.data.showSubmitOption && aomJobSrv.data.features.canSubmit },
            { text: aomResourcesSrv.data.resources.Common.Attachments, key: 'Attachments', show: true },
            { text: aomResourcesSrv.data.resources.AomActionSheetOptions.JobList, key: 'Back', show: aomJobSrv.data.selectedJob.Paging.selectedPage !== aomResourcesSrv.data.resources.Job.FirstPageName },
            { text: aomResourcesSrv.data.resources.AomActionSheetOptions.Drive, key: 'Drive', show: aomJobSrv.data.selectedJob.jsProperties.workLocationSet },
            { text: aomResourcesSrv.data.resources.Common.Save, key: 'Save', show: true },
            { text: aomResourcesSrv.data.resources.Job.Revert, key: 'Revert', show: true }
          ].filter(function (option) {
            return option.show;
          });

          $ionicActionSheet.show({
            buttons: options,
            titleText: aomResourcesSrv.data.resources.AomActionSheetOptions.Title,
            cancelText: aomResourcesSrv.data.resources.Common.Cancel,
            cancel: function () {
              return true;
            },
            buttonClicked: function (index, object) {
              if (object.key === 'Submit') {
                $scope.submitSelected();
              } else if (object.key === 'Attachments') {
                $scope.showAttachments();
              } else if (object.key === 'Back') {
                backToJobs();
              } else if (object.key === 'Drive') {
                aomComponentsSrv.launchNavigator(aomJobSrv.data.selectedJob.WorkLocation.Latitude, aomJobSrv.data.selectedJob.WorkLocation.Longitude);
              } else if (object.key === 'BridgeLink') {
                window.aomURLOpen(aomResourcesSrv.data.resources.Bridge.BridgeLink);
              } else if (object.key === 'Revert') {
                revertJob();
              } else if (object.key === 'Save') {
                saveAndExit();
              }
              return true;
            },
            cssClass: 'job-options'
          });
        }

        $scope.showOptions = function () {
          if ($scope.data.currentState === constants.viewStates.job) {
            showJobOptions();
          } else if ($scope.data.currentState === constants.viewStates.attachments) {
            $scope.$broadcast('AttachmentOptions');
          } else if ($scope.data.currentState === constants.viewStates.jobMap) {
            $scope.$broadcast('MapOptions');
          }
        };

        $scope.submitSelected = function () {
          if ($scope.data.pageValid === true) {
            if (aomResourcesSrv.data.resources.appConfig.ConfirmSubmit) {
              aomSubmitCheckSrv.data.submitting = true;
              showIonicPopUp(aomResourcesSrv.data.resources.Common.Submit, aomResourcesSrv.data.resources.Job.SubmitMessage, true);
            } else {
              submitAndSetStatus(constants.status.pending);
            }
          }
        };

        $scope.page = function (field, $event) {
          $event.stopPropagation();
          if ($scope.data.pageValid) {
            $scope.data.animation = 'next-page';
            aomJobSrv.data.selectedJob.Paging.pageHistory.push(aomJobSrv.data.selectedJob.Paging.selectedPage);
            aomJobSrv.data.selectedJob.Paging.selectedPage = field.Data.replace('aompage://', '');
            setCalcFlags();
            aomJobSrv.setShowSubmitFlags();
            scrollToTop();
          }
        };

        function getPage(pageName) {
          var pages = aomJobSrv.data.selectedJob.Paging.Pages;

          for (var i = 0; i < pages.length; i++) {
            if (pageName === pages[i].Name)
              return pages[i];
          }

          return null;
        }

        $scope.showSubHeading = function () {
          return ($scope.data.currentState === constants.viewStates.job) && (aomJobSrv.data.selectedJob.Paging.Pages.length > 1);
        };

        $scope.getCurrentPageName = function () {
          var pageName = aomJobSrv.data.selectedJob.Paging.selectedPage;

          var page = getPage(pageName);
          if (page) pageName = page.DisplayName;

          return pageName;
        };

        $scope.isFirstPage = function () {
          return aomJobSrv.data.selectedJob.Paging.selectedPage === aomResourcesSrv.data.resources.Job.FirstPageName;
        };

        $scope.$watch('data.aomJobSrvData.aomSqlMapSrvData.isSQLRunning', function () {
          setCalcFlags();
        });

        // $scope.$watch('shared.keyboardUp', function () {
        //   var scrollBy = $scope.shared.keyboardUp ? $scope.data.paddingHeight : $scope.data.paddingHeight * -1;
        //   $ionicScrollDelegate.$getByHandle('job-table').scrollBy(0,  scrollBy, true);
        // });

        $scope.showJob = function () {
          $scope.data.currentState = constants.viewStates.job;
          aomJobSrv.setField({});
        };

        $scope.showAttachments = function () {
          $scope.data.currentState = constants.viewStates.attachments;
          setAttachmentSrvData(aomJobSrv.data.selectedJob.Attachments, { Title: aomResourcesSrv.data.resources.Common.Attachments, Type: 'All', readonly: false, isField: false });
          aomJobSrv.setField({});
        };

        $scope.$on('$ionicView.enter', function () {
          scrollToSelectedField();
        });

        $timeout(function () {
          $scope.data.limit = 9999;
          removeJobWatch = $scope.$watch('data.aomJobSrvData.selectedJob', function () {
            if (aomJobSrv.data.initComplete) {
              aomJobSrv.data.changesMade = true;
              aomAutoSaveSrv.saveTempJob();
            }
          }, true);
          initJob();
        });

        $scope.$on('$destroy', function () {
          aomJobSrv.data.activeJobId = null;
          if (deregister) deregister();
        });

      },
      templateUrl: 'App/Components/Job/job.html'
    };

  });