﻿'use strict';
angular.module('aom.directives')
  .directive('aomJobs', function ($location, $state, $timeout, $ionicScrollDelegate, $ionicSideMenuDelegate, $ionicPopup, $filter, aomJobsSrv, aomResourcesSrv, aomGoogleMapsSrv, aomJobSrv, aomPopupSrv, constants, aomNotificationSrv, aomStyleSrv, aomNetworkSrv, aomOfflineDatabasesSrv, aomStateSrv) {
    return {
      scope: {
        type: '@',
        onSelect: '&'
      },
      controller: function ($scope) {

        var topScrollPosition = 0,
          scrollTimeout = null,
          searchingTimeout = null;

        $scope.getSortOrderClass = function getSortOrderClass() {
          // When restarting the system when the app comes online after an offline logon,
          // there is a delay before the appConfig is reloaded, so we need to check 
          // that the referenced resources have actually been initialised, if not simply return
          if (aomResourcesSrv.data.resources.appConfig) {
            return aomResourcesSrv.data.resources.appConfig.SortControl ? 'descending' : 'ascending';
          }
        };

        $scope.data = {
          resources: aomResourcesSrv.data.resources,
          jobServiceData: aomJobsSrv.data,
          mapData: aomGoogleMapsSrv.data,
          rowClass: 'row-spacing-md',
          notificationSrvData: aomNotificationSrv.data,
          applications: null,
          hideSearch: false,
          stateSrvData: aomStateSrv.data,
          viewStates: constants.viewStates,
          count: 0
        };


        switch ($scope.type) {
          case constants.viewStates.inbox:
            $scope.data.content = aomJobsSrv.data.filteredJobs;
            $scope.filterJobs = function filterJobs(job) {
              return job.jsProperties.status !== 'pending' && job.jsProperties.status !== 'deleted';
            };
            $scope.data.showMapIcon = true;
            break;
          case constants.viewStates.compose:
            $scope.data.content = aomJobsSrv.data.filteredTemplates;
            $scope.filterJobs = function filterJobs(job) {
              return job;
            };
            $scope.data.showMapIcon = false;
            break;
          case constants.viewStates.outbox:
            $scope.data.content = aomJobsSrv.data.filteredJobs;
            $scope.filterJobs = function filterJobs(job) {
              return job.jsProperties.status === 'pending' && job.jsProperties.status !== 'deleted';
            };
            $scope.data.showMapIcon = false;
            break;
        }

        $scope.$on(constants.broadcastEvents.selectJobMarker, function (event, job) {
          var id = 'job' + job.jobId;
          $location.hash(id);
          $ionicScrollDelegate.anchorScroll();
          $scope.data.jobServiceData.selectedJobId = job.jobId;
          $scope.data.jobServiceData.selectedMapJobId = job.jobId;
        });

        $scope.$on('OpenSelectedJob', function (event, job) {
          const selectedJob = aomJobsSrv.data.jobs.filter(aJob => aJob.Id === job.jobId)[0];
          $scope.edit(selectedJob);
        });


        // TODO Like this after data reset but not if there are pending jobs to be sent.  Not sure what the best option is...
        $timeout(function () {
          $scope.data.animation = 'slide-in-left';
        }, 5000);

        $scope.data.groups = ($scope.data.resources.derived.SelectedSort.Code === 'Code') ? aomResourcesSrv.data.lastUser.groups : [{ Reference: '' }];
        $scope.data.rowClass = aomStyleSrv.getRowClass();


        var ionicBeforeEnter = $scope.$on('$ionicView.beforeEnter', function (event, viewData) {
          viewData.enableBack = false;
        });

        $scope.$on('$ionicView.afterEnter', function () {
          $timeout(function () {
            $scope.data.limit = 9999;
          });
        });

        $scope.$on('$destroy', function () {
          ionicBeforeEnter();
        });

        // aomJobsSrv.jobsViewLoaded();
        showVersionUpgradePopUp();

        function showIonicPopup(template, title) {
          aomPopupSrv.addPopup('jobsGeneric', $ionicPopup.show({
            template: template,
            title: title,
            scope: $scope,
            buttons: [
              {
                text: aomResourcesSrv.data.resources.Common.Okay,
                type: 'button-positive'
              }
            ]
          }));
        }

        function compareVersion(resourceVersion, appVersion) {
          var appVersionParts = appVersion.split('.').map(Number);
          var resourceVersionParts = resourceVersion.split('.').map(Number);

          for (var index = 0; index < appVersionParts.length; index++) {
            if (resourceVersionParts[index] > appVersionParts[index]) {
              return true;
            }
            if (resourceVersionParts[index] < appVersionParts[index]) {
              return false;
            }
          }
          return false;
        }

        function showVersionUpgradePopUp() {
          if (aomJobsSrv.data.showUpgradePrompt && compareVersion(aomResourcesSrv.data.resources.RequiredAppVersion, constants.common.appVersion)) {
            showIonicPopup(aomResourcesSrv.data.resources.Common.Version + aomResourcesSrv.data.resources.RequiredAppVersion + aomResourcesSrv.data.resources.Common.UpgradeText, aomResourcesSrv.data.resources.Common.UpgradeTitle);
            aomJobsSrv.data.showUpgradePrompt = false;
          }
        }

        function scrolled() {
          if (!$ionicScrollDelegate.$getByHandle('jobsScroll')) return;

          var newScrollPosition = $ionicScrollDelegate.$getByHandle('jobsScroll').getScrollPosition().top;
          var keyboardIsVisible = document.activeElement === document.getElementById('jobSearch');
          if ($scope.data.jobServiceData.searchQuery === undefined || $scope.data.jobServiceData.searchQuery === '' && !keyboardIsVisible) {
            if (newScrollPosition > 30 && newScrollPosition - topScrollPosition > 150) {
              $scope.data.hideSearch = true;
              topScrollPosition = newScrollPosition;
            }
          }
          if (topScrollPosition - newScrollPosition > 50) {
            $scope.data.hideSearch = false;
            topScrollPosition = newScrollPosition;
          }
          scrollTimeout = null;
        }

        function initJob(selectedJob) {
          aomJobSrv.data.selectedJob = selectedJob;
          aomJobsSrv.data.selectedJobId = selectedJob.Id;
          aomJobsSrv.data.selectedMapJobId = null;
          aomJobSrv.data.features = {
            readonly: false,
            linkToBridge: false,
            runSql: true,
            canSubmit: true
          };
          $timeout(function () {
            aomJobSrv.init()
              .then(function () {
                $scope.onSelect();
                $scope.type !== constants.viewStates.outbox ? aomStateSrv.setRight(constants.viewStates.job) : angular.noop();
              });
          }, 100);
        }

        aomJobsSrv.setupDraftJobs();

        $scope.scrolled = function () {
          if (scrollTimeout === null) {
            scrollTimeout = $timeout(scrolled, 50);
          }
        };

        $scope.edit = function (selectedJob) {
          if (selectedJob.jsProperties.status !== constants.status.pending && selectedJob.jsProperties.status !== constants.status.updating) {
            if (aomOfflineDatabasesSrv.data.importing && selectedJob.SqlMap.length > 0) {
              aomPopupSrv.confirm('ImportPending', {
                title: aomResourcesSrv.data.resources.Jobs.PendingImportAlertTitle,
                template: aomResourcesSrv.data.resources.Jobs.PendingImportAlertTemplate,
                onYes: function () {
                  initJob(selectedJob);
                },
                onNo: function () {
                }
              });
            } else {
              initJob(selectedJob);
            }
          }
        };

        $scope.clearSearch = function () {
          $scope.data.jobServiceData.searchQuery = '';
          $scope.data.jobServiceData.setSearchQuery = angular.copy($scope.data.jobServiceData.searchQuery);
          $scope.data.hideSearch = true;
        };

        $scope.toggleSearch = function () {
          $scope.data.hideSearch = !$scope.data.hideSearch;
        };

        $scope.searchChanged = function () {
          $scope.data.animation = '';
          if (searchingTimeout !== null) {
            $timeout.cancel(searchingTimeout);
          }
          searchingTimeout = $timeout(function () {
            $scope.data.animation = 'slide-in-left';
          }, 1000);
          $timeout(function () {
            $scope.data.jobServiceData.setSearchQuery = angular.copy($scope.data.jobServiceData.searchQuery);
          });
          $ionicScrollDelegate.$getByHandle('jobsScroll').anchorScroll(true);
        };

        $scope.showMap = function () {
          $timeout(function () {
            if (aomNetworkSrv.online()) {
              aomJobsSrv.data.selectedJobId = -1;
              aomJobsSrv.data.selectedMapJobId = -1;
              aomStateSrv.setRight(!aomStateSrv.data.showingMap ? constants.viewStates.map : constants.viewStates.empty);
            } else {
              aomNotificationSrv.notify(aomResourcesSrv.data.resources.Errors.NoInternet);
            }
          }, 30);
        };

        $scope.changeContents = function () {
          $scope.data.animation = '';
          $timeout(function () {
            $state.go($scope.content.rightButtonState);
          }, 30);
        };

        $scope.showInfo = function (selectedJob, $event) {
          $event.stopPropagation();
          var infoHTML = '';
          infoHTML += selectedJob.WorkLocation.Address === null ? '' : aomResourcesSrv.data.resources.Common.WorkLocationAddress + ': ' + selectedJob.WorkLocation.Address + '<br/>';
          infoHTML += selectedJob.Application === null ? '' : aomResourcesSrv.data.resources.Common.Application + ': ' + selectedJob.Application + '<br/>';
          infoHTML += selectedJob.Reference === null ? '' : aomResourcesSrv.data.resources.Common.Reference + ': ' + selectedJob.Reference + '<br/>';
          infoHTML += selectedJob.DateCreated === null ? '' : aomResourcesSrv.data.resources.Common.DateCreated + ': ' + $filter('aomLongDateFtr')(selectedJob.DateCreated) + '<br/>';
          infoHTML += selectedJob.DueDate === '' ? '' : aomResourcesSrv.data.resources.Common.DueDate + ': ' + $filter('aomLongDateFtr')(selectedJob.DueDate);

          showIonicPopup(infoHTML, selectedJob.Title);
        };

        $scope.showSideMenu = function () {
          $ionicSideMenuDelegate.toggleLeft();
        };

        $scope.sortItems = function sortItems() {
          aomResourcesSrv.data.resources.appConfig.SortControl = !aomResourcesSrv.data.resources.appConfig.SortControl;
        };

        $scope.hideUpdate = function () {
          aomNotificationSrv.data.displayUpdateMessageOnHeader = false;
        };

        $scope.getCount = function () {
          $scope.data.count = $scope.data.groups.reduce((total, group) => {
            total += group.items === undefined ? 0 : group.items.length; return total;
          }, 0);
          return $scope.data.count;
        };

        $scope.getLabel = function () {
          if ($scope.type === constants.viewStates.compose) {
            return $scope.data.count === 1
              ? aomResourcesSrv.data.resources.Jobs.Template
              : aomResourcesSrv.data.resources.Jobs.Templates;
          } else {
            return $scope.data.count === 1
              ? aomResourcesSrv.data.resources.Jobs.Job
              : aomResourcesSrv.data.resources.Jobs.Jobs;
          }
        };

        $scope.getFilteredResultsCount = function getFilteredResultsCount() {
          const total = $scope.data.groups.reduce((total, group) => {
            total += group.filterResults === undefined ? 0 : group.filterResults.length; return total;
          }, 0);
          if (aomJobsSrv.data.searchQuery) {
            $scope.data.titleClass = 'title-search';
            return total + ' of ';
          } else {
            $scope.data.titleClass = 'title-no-search';
            return '';
          }
        };

        $scope.getStatus = function getStatus(job) {
          if ($scope.type === 'jobs' || $scope.type === 'templates') {
            var filteredJob = aomJobsSrv.data.drafts.filter(function (aJob) {
              return aJob.Id === job.Id;
            });
            return filteredJob.length > 0 ? constants.status.drafts : $scope.type === 'jobs' ? job.jsProperties.status : '';
          } else {
            return '';
          }
        };

        $scope.$on('$destroy', function () {
          aomJobsSrv.data.selectedJobId = -1;
          aomJobsSrv.data.selectedMapJobId = -1;
        });
      },
      templateUrl: 'App/Components/Jobs/jobs.html'
    };
  });