'use strict';
var app = angular.module('aom');

app.constant('constants',
  {
    anyFileFromDeviceTypes: ['application/pdf',
      'application/msword', 'application/vnd.openxmlformats-officedocument.wordprocessingml.document',
      'application/vnd.ms-excel', 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet',
      'application/vnd.ms-powerpoint', 'application/vnd.openxmlformats-officedocument.presentationml.presentation'],
    common: {
      appName: 'AllOnMobile V4',
      appVersion: '5.0.1.37',
      logoFolder: 'aomLogo',
      defaultLogo: 'images/logo.png',
      aomAttachmentsDirectory: 'Attachments',
      aomAdhocAttachmentsDirectory: 'Attachments/Adhoc',
      logsFolder: 'logs',
      showAllCode: 'Show All',
      sharedFiles: 'SharedFiles', //Used by iOS for Shared documents
      fileNameSeperator: 'AOMSeperator',
      applicationResourceName: 'Applications'
    },
    dateFormat: 'DD/MM/YYYY',
    timeFormat: 'HH:mm',
    dateSettings: {
      theme: 'ios',
      display: 'bottom',
      min: new Date(1900, 1, 1),
      max: new Date(2040, 1, 1),
      buttons: [
        'set',
        'cancel',
        'clear'
      ]
    },
    extensionsToMimeTypes: {
      '.jpg': 'image/jpg',
      '.png': 'image/png',
      '.mov': 'video/mov',
      '.mp4': 'video/mp4',
      '.m4v': 'video/m4v',
      '.3gp': 'video/3gpp',
      '.pdf': 'application/pdf',
      '.doc': 'application/msword',
      '.docx': 'application/vnd.openxmlformats-officedocument.wordprocessingml.document',
      '.xls': 'application/vnd.ms-excel',
      '.xlsx': 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet',
      '.ppt': 'application/vnd.ms-powerpoint',
      '.pptx': 'application/vnd.openxmlformats-officedocument.presentationml.presentation'
    },
    fieldTypes: {
      textbox: 0,
      dropdown: 1,
      checkbox: 2,
      radio: 3,
      date: 4,
      time: 5,
      photoVideo: 6,
      currentPosition: 7,
      divider: 8,
      signature: 9,
      barcode: 10,
      attachment: 11,
      action: 12,
      freehand: 13,
      toggle: 14,
      zebraPrinting: 16,
      pageBreak: 19,
      submitbutton: 21,
      pagebutton: 22,
      counter: 23,
      searchIndex: 24,
      docSearch: 25,
      dateTime: 26,
      dateTimeDiff: 27
    },
    keyboardTypes: {
      '0': 'text',
      '2': 'url',
      '3': 'tel',
      '4': 'email'
    },
    logging: {
      logToConsole: false,
      dbLogsToKeep: 100,
      levels: {
        low: 1,
        medium: 2,
        high: 3,
        user: 4,
        none: 5
      },
      severities: {
        debug: 'debug',
        info: 'info',
        warn: 'warn',
        err: 'error'
      },
      fileLogs: {
        high: {
          id: 'high',
          dir: 'logs/high',
          minRecords: 5000
        },
        medium: {
          id: 'medium',
          dir: 'logs/medium',
          minRecords: 1000
        }
      }
    },
    mimeTypes: {
      image: ['image/png', 'image/jpeg', 'image/jpg'],
      video: ['video/m4v', 'video/mov', 'video/mp4', 'video/3gpp'],
      pdf: ['application/pdf'],
      document: ['application/msword', 'application/vnd.openxmlformats-officedocument.wordprocessingml.document'],
      excel: ['application/vnd.ms-excel', 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet'],
      powerpoint: ['application/vnd.ms-powerpoint', 'application/vnd.openxmlformats-officedocument.presentationml.presentation']
    },
    offlineDatabases: {
      unavailable: 'unavailable_'
    },
    polling: [
      'sendAttachments',
      'getJobs',
      'sendJobs',
      'sendAppSettings',
      'housekeepLogs',
      'sendLocation'
    ],
    states: {
      firstState: 'app.Main',
      mediaList: 'app.MediaList',
      canvas: 'app.ImageEditor'
    },
    viewPanels: {
      left: 'main',
      right: 'details'
    },
    viewStates: {
      empty: '',
      inbox: 'jobs',
      compose: 'templates',
      outbox: 'pending',
      order: 'order',
      settings: 'settings',
      logout: 'logon',
      map: 'map',
      job: 'job',
      jobMap: 'map',
      attachments: 'attachments'
    },
    menuSizes: {
      compact: { width: 190, class: 'compact' },
      standard: { width: 90, class: 'standard' }
    },
    status: {
      new: 'new',
      opened: '',
      pending: 'pending',
      updated: 'updated',
      updating: 'updating',
      deleted: 'deleted',
      drafts: 'drafts'
    },
    sql: {
      tables: {
        jobs: 'CREATE TABLE IF NOT EXISTS Jobs (id text primary key, detail_json text)',
        templates: 'CREATE TABLE IF NOT EXISTS Templates (id text primary key, detail_json text)',
        attachments: 'CREATE TABLE IF NOT EXISTS Attachments (jobId text, fileName text, remoteHref text, mimeType text, saveState text, PRIMARY KEY (jobId, fileName))',
        recovery: 'CREATE TABLE IF NOT EXISTS Recovery (id text primary key, detail_json text)',
        lastUser: 'CREATE TABLE IF NOT EXISTS LastUser (id integer primary key, user_json text)',
        offlineDatabases: 'CREATE TABLE IF NOT EXISTS OfflineDatabases (id TEXT primary key, database_json TEXT)',
        logs: 'CREATE TABLE IF NOT EXISTS Logs (id integer primary key, log_json text, severity text, timestamp DATETIME, uploading text)',
        logFiles: 'CREATE TABLE IF NOT EXISTS LogFiles (id text primary key, fileName text)',
        rootLink: 'CREATE TABLE IF NOT EXISTS RootLink (id text primary key, link text)'
      },
      adHoc: {
        migration1: 'ALTER TABLE Attachments ADD COLUMN saveState text',
        // The below line should be removed from the internal release builds but set to the live url for the production build
        // rootLink: 'INSERT OR REPLACE INTO RootLink (id, link) values (1, \'https://privateapi.allonmobile.com/api/rest/links\')'
        migration2a: 'ALTER TABLE Attachments ADD COLUMN timestamp DATETIME',
        migration2b: 'ALTER TABLE Attachments ADD COLUMN source text'
      }
    },
    timeSettings: {
      theme: 'ios',
      display: 'bottom',
      timeFormat: 'HH:ii:ss',
      timeWheels: 'HHii',
      buttons: [
        'set',
        'cancel',
        'clear'
      ]
    },
    dateAndTimeTimeSettings: {
      theme: 'ios',
      display: 'bottom',
      timeFormat: 'HH:ii:ss',
      timeWheels: 'HHiiss',
      buttons: [
        'set',
        'cancel',
        'clear'
      ]
    },
    broadcastEvents: {
      selectJob: 'selectJob',
      selectJobMarker: 'selectJobMarker',
      selectSetting: 'selectSetting',
      showOfflineDatabaseOptions: 'showOfflineDatabaseOptions'
    },
  });
