/* global StatusBar */
/* global cordova */
/* global angular */

var callback = function () {
  var app = angular.module('aom');
  var initInjector = angular.injector(['ng']);
  var $http = initInjector.get('$http');

  return $http.get('appconfig.json')
    .then(function (response) {
      app.constant('appConfig', response.data);
    })
    .then(function bootstrapApplication() {
      angular.element(document).ready(function () {
        angular.bootstrap(document, ['aom']);
      });
    });
};

if (window.cordova) {
  document.addEventListener('deviceready', callback, false);
} else {
  window.addEventListener('load', callback, false);
}

function handleOpenURL(url) {
  window.localStorage.setItem('external_load', url);
  var evt = new Event('aomv4ExternalLoad');
  document.dispatchEvent(evt);
}

var keyboardState = null;

window.aomURLOpen = function (link) {
  window.aomURLClicked = true;
  var ref = window.open(link, '_system', 'location = yes');
  ref.addEventListener('loaderror', function () {
    window.aomURLClicked = false;
  }, false);

};

angular.module('aom.controllers', []);
angular.module('aom.services', []);
angular.module('aom.directives', []);
angular.module('aom.constants', []);

angular.module('aom', ['ionic', 'ngCordova', 'leaflet-directive', 'mobiscroll-datetime', 'aom.directives', 'aom.controllers', 'aom.services', 'aom.constants'])

  .run(function ($ionicPlatform, $rootScope, $timeout, $ionicHistory, $ionicLoading, aomMigrationSrv) {
    if (ionic.Platform.isIOS()) {
      setTimeout(function () {
        if (navigator.splashscreen) {
          navigator.splashscreen.hide();
        }
      }, 1000);
    }
    $rootScope.accessoryBarDisplayStyle = 'none';

    $ionicPlatform.ready(function () {
      // Hide the accessory bar by default (remove this to show the accessory bar above the keyboard
      // for form inputs)
      // This causes issues on windows don't do ths for windows
      if (window.cordova && window.cordova.plugins && window.cordova.plugins.Keyboard && !ionic.Platform.isWindowsPhone() && !ionic.Platform.isEdge()) {
        console.log('Hide accessory bar');
        cordova.plugins.Keyboard.hideKeyboardAccessoryBar(false);
        cordova.plugins.Keyboard.disableScroll(true);
      }

      if (ionic.Platform.isAndroid()) {
        StatusBar.backgroundColorByName('gray');
        ionic.Platform.fullScreen(false, true);
      } else {
        StatusBar.styleDefault();
      }

      if (window.cordova && window.MobileAccessibility) {
        window.MobileAccessibility.usePreferredTextZoom(false);
      }

      if (ionic.Platform.isWindowsPhone()) {
        $ionicPlatform.registerBackButtonAction(function (evt) {
          if (evt && evt.type === 'backclick') {
            $ionicHistory.goBack();
            return true;
          }
        }, 100);

        WinJS.Application.onbackclick = function (evt) {
          $ionicPlatform.hardwareBackButtonClick(evt);
          return true;
        };
      }

      $rootScope.isIOS = ionic.Platform.isIOS();

      // Remove the back arrow on Windows 
      if (navigator.appVersion.indexOf('MSAppHost/3.0') !== -1) { // Windows 10 UWP (PC/Tablet/Phone)
        var navigationManager = Windows.UI.Core.SystemNavigationManager.getForCurrentView();
        navigationManager.appViewBackButtonVisibility = Windows.UI.Core.AppViewBackButtonVisibility.collapsed;
      }

      aomMigrationSrv.init();

    });

    $rootScope.$on('$stateChangeStart', function (event, nextState, toParams, fromState) {
      if (window.aomURLClicked) {
        window.aomURLClicked = false;
        if (nextState.name === 'logon' && fromState.name === 'app.Job') {
          event.preventDefault();
        }
      }
      if (!window.cordova) {
        var injector = angular.element(document.body).injector();
        var initSrv = injector.get('aomInitSrv');
        if (nextState.name !== 'logon' && !initSrv.initialized) {
          var $state = injector.get('$state');
          initSrv.init().then(function () {
            $state.go(nextState.name);
          });
        }
      }
      nextState.resolve.init = function (aomInitSrv) {
        return aomInitSrv.init();
      };
    });

    $rootScope.$on('$stateChangeSuccess', function () {
      $timeout(function () {
        $ionicLoading.hide();
        // console.log('State Change End' + moment().format('HH mm ss SS'));
      }, 500);
    });
  })

  .config(function ($compileProvider, $httpProvider, $ionicConfigProvider, $logProvider) {
    $httpProvider.interceptors.push('aomAuthInterceptorSrv');

    $ionicConfigProvider.scrolling.jsScrolling(false);
    $ionicConfigProvider.views.maxCache(0);
    $ionicConfigProvider.views.swipeBackEnabled(false);

    $ionicConfigProvider.platform.android.views.transition('ios');
    $ionicConfigProvider.platform.android.tabs.style('standard').position('bottom');
    $ionicConfigProvider.platform.android.navBar.alignTitle('center').positionPrimaryButtons('left');
    $ionicConfigProvider.platform.android.form.toggle('large');

    $compileProvider.imgSrcSanitizationWhitelist(/^\s*(https?|file|blob|ms-appx-web|ms-appdata|cdvfile|content|ionic):|data:image\//);
    $compileProvider.aHrefSanitizationWhitelist(/^\s*(https?|ftp|mailto|file|ms-appdata|chrome-extension|bingmaps|geo|blob|ionic):/);
    $httpProvider.interceptors.push('aomTimeoutInterceptorSrv');

    if (!ionic.Platform.isAndroid() && !ionic.Platform.isIOS()) {
      $ionicConfigProvider.platform.default.spinner.icon('android');
    }
    $logProvider.debugEnabled(false);
  });
